/*
* Copyright 2004 - 2009 by dronten@gmail.com
*
* This source is distributed under the terms of the GNU PUBLIC LICENSE version 3
* http://www.gnu.org/licenses/gpl.html
*/

package com.googlepages.dronten.jtagger.gui.tracktab;

import com.googlepages.dronten.jtagger.JTagger;
import com.googlepages.dronten.jtagger.audio.AudioTrack;
import com.googlepages.dronten.jtagger.gui.ComponentFactory;
import com.googlepages.dronten.jtagger.model.TrackModel;
import com.googlepages.dronten.jtagger.thread.AudioThread;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


/**
 * List all mp3 songs from the TrackModel object.
 * When user selects a song display tags about the track in the TrackOptions panel.
 */
public class TrackTablePanel extends JPanel implements ActionListener {
    private static final long   serialVersionUID = 1;
    private JButton             aDeleteButton = null;
    private JTable              aTable = null;
    private TrackOptionsPanel   aTrackOptionsPanel = null;


    /**
     * @param trackOptions Track options panel
     */
    public TrackTablePanel(TrackOptionsPanel trackOptions) {
        super();
        setLayout(new BorderLayout());
        aTrackOptionsPanel = trackOptions;
        aTable = new JTable(new TrackModel());
        JScrollPane centerPanel = new JScrollPane();
        JPanel southPanel = new JPanel();

        southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.Y_AXIS));
        setBorder(BorderFactory.createEmptyBorder(5, 3, 5, 3));

        aTable.getColumnModel().getColumn(0).setMinWidth(50);
        aTable.getColumnModel().getColumn(0).setPreferredWidth(50);
        aTable.getColumnModel().getColumn(0).setMaxWidth(100);

        aTable.getColumnModel().getColumn(1).setMinWidth(100);
        aTable.getColumnModel().getColumn(1).setPreferredWidth(500);
        aTable.getColumnModel().getColumn(1).setMaxWidth(1000);

        aTable.getColumnModel().getColumn(2).setMinWidth(50);
        aTable.getColumnModel().getColumn(2).setPreferredWidth(50);
        aTable.getColumnModel().getColumn(2).setMaxWidth(100);

        aTable.getTableHeader().setToolTipText("<html>Click table header to sort rows<br>ctrl-click to reverse sort order</html>");
        aTable.setToolTipText("Double click to edit filename or track number");
        aTable.setShowGrid(false);
        aTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        aTable.getTableHeader().addMouseListener(getTrackModel().getMouseHandler());

        southPanel.add(Box.createRigidArea(new Dimension(0, 1)));
        JPanel buttonPanel = ComponentFactory.createPanelWithTwoColumns(1, 2);

        aDeleteButton = ComponentFactory.createButton("Delete", "", this);
        aDeleteButton.setEnabled(false);
        buttonPanel.add(aDeleteButton);

        // When user selects a file set mp3 info in TrackOptions
        ListSelectionModel rowSM = aTable.getSelectionModel();
        rowSM.addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (e.getValueIsAdjusting()) {
                    return;
                }

                aDeleteButton.setEnabled(false);
                ListSelectionModel lsm = (ListSelectionModel) e.getSource();

                if (lsm.isSelectionEmpty() == false) {
                    int         selectedRow = lsm.getMinSelectionIndex();
                    TrackModel  model = (TrackModel) aTable.getModel();
                    AudioTrack  track = model.getTrack(selectedRow);

                    if (track != null) {
                        aTrackOptionsPanel.setTrack(track);
                    }

                    aDeleteButton.setEnabled(true);
                    JTagger.get().selectRow(selectedRow);
                }
                else {
                    aTrackOptionsPanel.setTrack(null);
                }
            }
        });

        southPanel.add(buttonPanel);
        centerPanel.getViewport().setView(aTable);
        add(centerPanel, BorderLayout.CENTER);
        add(southPanel, BorderLayout.SOUTH);
    }


    /**
     * Take care of button events.
     *
     * @param actionEvent
     */
    @Override public void actionPerformed(ActionEvent actionEvent) {
        try {
            if (actionEvent.getSource() == aDeleteButton) {
                ListSelectionModel lsm = aTable.getSelectionModel();

                if (!lsm.isSelectionEmpty()) {
                    int selectedRow = lsm.getMinSelectionIndex();

                    AudioThread.get().throwIfPlaying();

                    Object[] options = {"Delete", "No"};

                    if (JOptionPane.showOptionDialog(null, "Are you sure about deleting this file?", "Warning", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[1]) == 0) {
                        getTrackModel().getTrack(selectedRow).delete();
                        getTrackModel().getAudioTracks().remove(selectedRow);
                        getTrackModel().fireTableDataChanged();

                        if (selectedRow == aTable.getModel().getRowCount()) {
                            selectedRow--;
                        }
                    }

                    JTagger.get().selectRow(selectedRow);
                }
                else {
                    assert false;
                }
            }
        }
        catch (Exception e) {
            JTagger.get().setMessage(e.getMessage(), true);
        }
    }


    /**
     * Get the TrackModel object.
     *
     * @return return TrackModel object
     */
    public TrackModel getTrackModel() {
        return (TrackModel) aTable.getModel();
    }


    /**
     * Select row in grid.
     *
     * @param row
     */
    public void selectRow(int row) {
        if (row >= 0) {
            ListSelectionModel lsm = aTable.getSelectionModel();
            lsm.setSelectionInterval(row, row);
        }
    }
}
